@@ -29,10 +29,8 @@
 extern "C" {
 #endif
 
-#include <sys/types.h>
 #include <sys/time.h>
 
-#include "asterisk/compiler.h"
 #include "asterisk/endian.h"
 #include "asterisk/linkedlists.h"
 
@@ -52,7 +50,7 @@ struct ast_codec_pref {
 	\arg \b VIDEO:	Video data, subclass is codec (AST_FORMAT_*)
 	\arg \b DTMF:	A DTMF digit, subclass is the digit
 	\arg \b IMAGE:	Image transport, mostly used in IAX
-	\arg \b TEXT:	Text messages
+	\arg \b TEXT:	Text messages and character by character (real time text)
 	\arg \b HTML:	URL's and web pages
 	\arg \b MODEM:	Modulated data encodings, such as T.38 and V.150
 	\arg \b IAX:	Private frame type for the IAX protocol
@@ -94,6 +92,7 @@ struct ast_codec_pref {
  *
  * \note It is important that the values of each frame type are never changed,
  *       because it will break backwards compatability with older versions.
+ *       This is because these constants are transmitted directly over IAX2.
  */
 enum ast_frame_type {
 	/*! DTMF end event, subclass is the digit */
@@ -127,18 +126,6 @@ enum ast_frame_type {
 enum {
 	/*! This frame contains valid timing information */
 	AST_FRFLAG_HAS_TIMING_INFO = (1 << 0),
-	/*! This frame came from a translator and is still the original frame.
-	 *  The translator can not be free'd if the frame inside of it still has
-	 *  this flag set. */
-	AST_FRFLAG_FROM_TRANSLATOR = (1 << 1),
-	/*! This frame came from a dsp and is still the original frame.
-	 *  The dsp cannot be free'd if the frame inside of it still has
-	 *  this flag set. */
-	AST_FRFLAG_FROM_DSP = (1 << 2),
-	/*! This frame came from a filestream and is still the original frame.
-	 *  The filestream cannot be free'd if the frame inside of it still has
-	 *  this flag set. */
-	AST_FRFLAG_FROM_FILESTREAM = (1 << 3),
 };
 
 /*! \brief Data structure associated with a single frame of data
@@ -150,7 +137,7 @@ struct ast_frame {
 	int subclass;				
 	/*! Length of data */
 	int datalen;				
-	/*! Number of 8khz samples in this frame */
+	/*! Number of samples in this frame */
 	int samples;				
 	/*! Was the data malloc'd?  i.e. should we free it when we discard the frame? */
 	int mallocd;				
@@ -195,9 +182,17 @@ struct ast_frame {
     for this purpose instead of having to declare one on the stack */
 extern struct ast_frame ast_null_frame;
 
-#define AST_FRIENDLY_OFFSET 	64	/*! It's polite for a a new frame to
-					  have this number of bytes for additional
-					  headers.  */
+/*! \brief Offset into a frame's data buffer.
+ *
+ * By providing some "empty" space prior to the actual data of an ast_frame,
+ * this gives any consumer of the frame ample space to prepend other necessary
+ * information without having to create a new buffer.
+ *
+ * As an example, RTP can use the data from an ast_frame and simply prepend the
+ * RTP header information into the space provided by AST_FRIENDLY_OFFSET instead
+ * of having to create a new buffer with the necessary space allocated.
+ */
+#define AST_FRIENDLY_OFFSET 	64	
 #define AST_MIN_OFFSET 		32	/*! Make sure we keep at least this much handy */
 
 /*! Need the header be free'd? */
@@ -261,9 +256,9 @@ extern struct ast_frame ast_null_frame;
 /*! G.722 */
 #define AST_FORMAT_G722		(1 << 12)
 /*! Unsupported audio bits */
-#define AST_FORMAT_AUDIO_UNDEFINED	((1 << 13) | (1 << 14) | (1 << 15))
-/*! Maximum audio format */
-#define AST_FORMAT_MAX_AUDIO	(1 << 15)
+#define AST_FORMAT_AUDIO_UNDEFINED	((1 << 13) | (1 << 14))
+/*! Raw 16-bit Signed Linear (16000 Hz) PCM */
+#define AST_FORMAT_SLINEAR16	(1 << 15)
 /*! Maximum audio mask */
 #define AST_FORMAT_AUDIO_MASK   ((1 << 16)-1)
 /*! JPEG Images */
@@ -278,9 +273,12 @@ extern struct ast_frame ast_null_frame;
 #define AST_FORMAT_H263_PLUS	(1 << 20)
 /*! H.264 Video */
 #define AST_FORMAT_H264		(1 << 21)
-/*! Maximum video format */
-#define AST_FORMAT_MAX_VIDEO	(1 << 24)
+/*! MPEG4 Video */
+#define AST_FORMAT_MP4_VIDEO	(1 << 22)
 #define AST_FORMAT_VIDEO_MASK   (((1 << 25)-1) & ~(AST_FORMAT_AUDIO_MASK))
+/*! T.140 Text format - ITU T.140, RFC 4351*/
+#define AST_FORMAT_T140		(1 << 25)
+#define AST_FORMAT_TEXT_MASK   (((1 << 30)-1) & ~(AST_FORMAT_AUDIO_MASK) & ~(AST_FORMAT_VIDEO_MASK))
 
 enum ast_control_frame_type {
 	AST_CONTROL_HANGUP = 1,		/*!< Other end has hungup */
@@ -301,7 +299,42 @@ enum ast_control_frame_type {
 	AST_CONTROL_HOLD = 16,		/*!< Indicate call is placed on hold */
 	AST_CONTROL_UNHOLD = 17,	/*!< Indicate call is left from hold */
 	AST_CONTROL_VIDUPDATE = 18,	/*!< Indicate video frame update */
+	_XXX_AST_CONTROL_T38 = 19,	/*!< T38 state change request/notification \deprecated This is no longer supported. Use AST_CONTROL_T38_PARAMETERS instead. */
 	AST_CONTROL_SRCUPDATE = 20,     /*!< Indicate source of media has changed */
+	AST_CONTROL_T38_PARAMETERS = 24, /*!< T38 state change request/notification with parameters */
+};
+
+enum ast_control_t38 {
+	AST_T38_REQUEST_NEGOTIATE = 1,	/*!< Request T38 on a channel (voice to fax) */
+	AST_T38_REQUEST_TERMINATE,	/*!< Terminate T38 on a channel (fax to voice) */
+	AST_T38_NEGOTIATED,		/*!< T38 negotiated (fax mode) */
+	AST_T38_TERMINATED,		/*!< T38 terminated (back to voice) */
+	AST_T38_REFUSED			/*!< T38 refused for some reason (usually rejected by remote end) */
+};
+
+enum ast_control_t38_rate {
+	AST_T38_RATE_2400 = 0,
+	AST_T38_RATE_4800,
+	AST_T38_RATE_7200,
+	AST_T38_RATE_9600,
+	AST_T38_RATE_12000,
+	AST_T38_RATE_14400,
+};
+
+enum ast_control_t38_rate_management {
+	AST_T38_RATE_MANAGEMENT_TRANSFERRED_TCF = 0,
+	AST_T38_RATE_MANAGEMENT_LOCAL_TCF,
+};
+
+struct ast_control_t38_parameters {
+	enum ast_control_t38 request_response;			/*!< Request or response of the T38 control frame */
+	unsigned int version;					/*!< Supported T.38 version */
+	unsigned int max_ifp; 					/*!< Maximum IFP size supported */
+	enum ast_control_t38_rate rate;				/*!< Maximum fax rate supported */
+	enum ast_control_t38_rate_management rate_management;	/*!< Rate management setting */
+	unsigned int fill_bit_removal:1;			/*!< Set if fill bit removal can be used */
+	unsigned int transcoding_mmr:1;				/*!< Set if MMR transcoding can be used */
+	unsigned int transcoding_jbig:1;			/*!< Set if JBIG transcoding can be used */
 };
 
 #define AST_SMOOTHER_FLAG_G729		(1 << 0)
@@ -350,6 +383,12 @@ enum ast_control_frame_type {
 /*! Explicitly enable or disable echo cancelation for the given channel */
 #define	AST_OPTION_ECHOCAN		8
 
+/* !
+ * Read-only. Allows query current status of T38 on the channel.
+ * data: ast_t38state
+ */
+#define AST_OPTION_T38_STATE		10
+
 struct oprmode {
 	struct ast_channel *peer;
 	int mode;
@@ -374,9 +413,9 @@ struct ast_option_header {
 
 /*! \brief Definition of supported media formats (codecs) */
 struct ast_format_list {
-	int visible;	/*!< Can we see this entry */
 	int bits;	/*!< bitmask value */
 	char *name;	/*!< short name */
+	int samplespersecond; /*!< Number of samples per second (8000/16000) */
 	char *desc;	/*!< Description */
 	int fr_len;	/*!< Single frame length in bytes */
 	int min_ms;	/*!< Min value */
@@ -399,9 +438,9 @@ struct ast_frame *ast_fralloc(char *source, int len);
 #endif
 
 /*!  
- * \brief Frees a frame 
+ * \brief Frees a frame or list of frames
  * 
- * \param fr Frame to free
+ * \param fr Frame to free, or head of list to free
  * \param cache Whether to consider this frame for frame caching
  */
 void ast_frame_free(struct ast_frame *fr, int cache);
@@ -415,6 +454,11 @@ void ast_frame_free(struct ast_frame *fr, int cache);
  * data malloc'd.  If you need to store frames, say for queueing, then
  * you should call this function.
  * \return Returns a frame on success, NULL on error
+ * \note This function may modify the frame passed to it, so you must
+ * not assume the frame will be intact after the isolated frame has
+ * been produced. In other words, calling this function on a frame
+ * should be the last operation you do with that frame before freeing
+ * it (or exiting the block, if the frame is on the stack.)
  */
 struct ast_frame *ast_frisolate(struct ast_frame *fr);
 
@@ -468,10 +512,24 @@ int ast_getformatbyname(const char *name);
  */
 char *ast_codec2str(int codec);
 
+/*! \name AST_Smoother 
+*/
+/*@{ */
+/*! \page ast_smooth The AST Frame Smoother
+The ast_smoother interface was designed specifically
+to take frames of variant sizes and produce frames of a single expected
+size, precisely what you want to do.
+
+The basic interface is:
+
+- Initialize with ast_smoother_new()
+- Queue input frames with ast_smoother_feed()
+- Get output frames with ast_smoother_read()
+- when you're done, free the structure with ast_smoother_free()
+- Also see ast_smoother_test_flag(), ast_smoother_set_flags(), ast_smoother_get_flags(), ast_smoother_reset()
+*/
 struct ast_smoother;
 
-struct ast_format_list *ast_get_format_list_index(int index);
-struct ast_format_list *ast_get_format_list(size_t *size);
 struct ast_smoother *ast_smoother_new(int bytes);
 void ast_smoother_set_flags(struct ast_smoother *smoother, int flags);
 int ast_smoother_get_flags(struct ast_smoother *smoother);
@@ -498,12 +556,16 @@ struct ast_frame *ast_smoother_read(struct ast_smoother *s);
 #define ast_smoother_feed_be(s,f) __ast_smoother_feed(s, f, 0)
 #define ast_smoother_feed_le(s,f) __ast_smoother_feed(s, f, 1)
 #endif
+/*@} Doxygen marker */
 
+struct ast_format_list *ast_get_format_list_index(int index);
+struct ast_format_list *ast_get_format_list(size_t *size);
 void ast_frame_dump(const char *name, struct ast_frame *f, char *prefix);
 
 /*! \page AudioCodecPref Audio Codec Preferences
+
 	In order to negotiate audio codecs in the order they are configured
-	in <channel>.conf for a device, we set up codec preference lists
+	in \<channel\>.conf for a device, we set up codec preference lists
 	in addition to the codec capabilities setting. The capabilities
 	setting is a bitmask of audio and video codecs with no internal
 	order. This will reflect the offer given to the other side, where
@@ -514,10 +576,16 @@ void ast_frame_dump(const char *name, struct ast_frame *f, char *prefix);
 	can't be transcoded and we just have to pick whatever is supported
 */
 
-/*! \brief Initialize an audio codec preference to "no preference" See \ref AudioCodecPref */
+/*! 
+ *\brief Initialize an audio codec preference to "no preference".
+ * \arg \ref AudioCodecPref 
+*/
 void ast_codec_pref_init(struct ast_codec_pref *pref);
 
-/*! \brief Codec located at a particular place in the preference index See \ref AudioCodecPref */
+/*! 
+ * \brief Codec located at a particular place in the preference index.
+ * \arg \ref AudioCodecPref 
+*/
 int ast_codec_pref_index(struct ast_codec_pref *pref, int index);
 
 /*! \brief Remove audio a codec from a preference list */
@@ -547,8 +615,9 @@ struct ast_format_list ast_codec_pref_getsize(struct ast_codec_pref *pref, int f
 /*! \brief Parse an "allow" or "deny" line in a channel or device configuration 
         and update the capabilities mask and pref if provided.
 	Video codecs are not added to codec preference lists, since we can not transcode
+	\return Returns number of errors encountered during parsing
  */
-void ast_parse_allow_disallow(struct ast_codec_pref *pref, int *mask, const char *list, int allowing);
+int ast_parse_allow_disallow(struct ast_codec_pref *pref, int *mask, const char *list, int allowing);
 
 /*! \brief Dump audio codec preference list into a string */
 int ast_codec_pref_string(struct ast_codec_pref *pref, char *buf, size_t size);
@@ -596,7 +665,7 @@ int ast_frame_slinear_sum(struct ast_frame *f1, struct ast_frame *f2);
  */
 static force_inline int ast_format_rate(int format)
 {
-	if (format == AST_FORMAT_G722)
+	if (format == AST_FORMAT_G722 || format == AST_FORMAT_SLINEAR16)
 		return 16000;
 
 	return 8000;
