@@ -23,31 +23,42 @@
 
 #ifndef _ASTERISK_SLINFACTORY_H
 #define _ASTERISK_SLINFACTORY_H
-#include <stdlib.h>
-#include <unistd.h>
-#include <string.h>
-
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
+#define AST_SLINFACTORY_MAX_HOLD 1280
+
 struct ast_slinfactory {
-	struct ast_frame *queue;
+	AST_LIST_HEAD_NOLOCK(, ast_frame) queue;
 	struct ast_trans_pvt *trans;
-	short hold[1280];
+	short hold[AST_SLINFACTORY_MAX_HOLD];
 	short *offset;
-	size_t holdlen;
-	int size;
-	int format;
+	size_t holdlen;			/*!< in samples */
+	unsigned int size;		/*!< in samples */
+	unsigned int format;
 };
 
 void ast_slinfactory_init(struct ast_slinfactory *sf);
-void ast_slinfactory_destroy(struct ast_slinfactory *sf);
-int ast_slinfactory_feed(struct ast_slinfactory *sf, struct ast_frame *f);
-int ast_slinfactory_read(struct ast_slinfactory *sf, short *buf, size_t bytes);
-		 
 
+/*!
+ * \brief Destroy the contents of a slinfactory
+ *
+ * \arg sf the slinfactory that is no longer needed
+ *
+ * This function will free any memory allocated for the contents of the
+ * slinfactory.  It does not free the slinfactory itself.  If the sf is
+ * malloc'd, then it must be explicitly free'd after calling this function.
+ *
+ * \return nothing
+ */
+void ast_slinfactory_destroy(struct ast_slinfactory *sf);
+
+int ast_slinfactory_feed(struct ast_slinfactory *sf, struct ast_frame *f);
+int ast_slinfactory_read(struct ast_slinfactory *sf, short *buf, size_t samples);
+unsigned int ast_slinfactory_available(const struct ast_slinfactory *sf);
+void ast_slinfactory_flush(struct ast_slinfactory *sf);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
